//+------------------------------------------------------------------+
//|                                   Copyright  2012, Ivan Kornilov|
//|                                          indicators\indiUtils.mqh|
//|                                   excelf@gmail.com, skype: excelf|
//|                                                   Indicator Utils|
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Ivan Kornilov. All rights reserved."
#property link "excelf@gmail.com"

//#include <indicators\indiUtils.mqh>


void indiUtils.resizeArray(int newSize, double &buffer[]) {
   ArraySetAsSeries(buffer, false);
   ArrayResize(buffer, newSize);
   ArraySetAsSeries(buffer, true);  
}

bool indiUtils.setManualBuffer(double &buffer[], double value, int shift) {
   buffer[Bars - 1 - shift] = value;
}

double indiUtils.getManualBuffer(double &buffer[], int shift) {
   return(buffer[Bars - 1 - shift]);
}

/**
 *        
 */
#define INDICATOR_SIGNAL_MODE_COLOR  0// 
#define INDICATOR_SIGNAL_MODE_CROSS  1//
#define INDICATOR_SIGNAL_MODE_LEVEL  2//.    .
#define INDICATOR_SIGNAL_MODE_ARROW  3//
#define INDICATOR_SIGNAL_TRADE_ALLOW 4//    EMPTY_VALUE  .    

double indiUtils.getBufferValue(string indicatorName, int tf, double arg1, double arg2, double arg3, double arg4, int bufferId, int paramCount, int i) {   
   switch(paramCount) {
        case 0:
            return(iCustom(NULL, tf, indicatorName, bufferId, i));
        case 1:
            return(iCustom(NULL, tf, indicatorName, arg1, bufferId, i));
        case 2:
            return(iCustom(NULL, tf, indicatorName, arg1, arg2, bufferId, i));     
        case 3:
            return(iCustom(NULL, tf, indicatorName, arg1, arg2, arg3, bufferId, i));
        case 4:
            return(iCustom(NULL, tf, indicatorName, arg1, arg2, arg3, arg4, bufferId, i));
        default: 
            return(0);
   }    
}

double indiUtils.normValue(double &buffer[], int normLimit, int i) {
   double max = buffer[ArrayMaximum(buffer, normLimit, i)];
   double min = buffer[ArrayMinimum(buffer, normLimit, i)];
   if(max == EMPTY_VALUE || min == EMPTY_VALUE) {
      return(EMPTY_VALUE);
   }
   double value = 0;
   if(max - min == 0) {
      value = 1;
   } else {
      value = (buffer[i] - min) / (max - min);
   }
   if (value > 1) {
      value = 1;
   } else if(value < 0) {
      value = 0;
   }
   return(value);
}

double indiUtils.normValueArray(double &buffer[], int normLimit, int i) {
   double max = buffer[ArrayMaximum(buffer, normLimit, Bars - 1 - i - normLimit)];
   double min = buffer[ArrayMinimum(buffer, normLimit, Bars - 1 - i - normLimit)];
   double value = 0;
   if(max - min == 0) {
      value = 1;
   } else {
      value = (indiUtils.getManualBuffer(buffer, i) - min) / (max - min);
   }
   if (value > 1) {
      value = 1;
   } else if(value < 0) {
      value = 0;
   }
   return(value);
   //return(indiUtils.getManualBuffer(buffer, i));
}

int indiUtils.barIdByTime(datetime time) {
    return(iBarShift(NULL, 0, time));
}

bool indiUtils.hasSymbol(string symbol) {
   return(iTime(symbol, 0, 0) != 0);
}

double indiUtils.getPrice(string symbol, int shift, bool div = false, int priceType = 0) {
   int sh = iBarShift(symbol, 0, Time[shift], true);
   double price = EMPTY_VALUE;
   if (sh < 0) {
      return(EMPTY_VALUE);
   } else {
      switch(priceType) {
         case 0: 
            price = iClose(symbol, 0, shift);
            break;
         case 1: 
            price = iHigh(symbol, 0, shift);
            break;
         case 2: 
            price = iLow(symbol, 0, shift);
            break;
         default:
            price = iClose(symbol, 0, shift);
            break;
      }
   }
   if(div) {
      if(price == 0) {
         price = 0.00001;
      }
      price = 1 /price; 
   }
   return(price);
}


double indiUtils.getFXIndex(string currName, string currensyName[], int shift){//USD EUR
   double res = 1.0;
   double opPrice;

   string symbol1;
   string symbol2;
   string symbol3;
   string symbol4;
  
   int currCount = ArraySize(currensyName);
   for(int ii = 0; ii < currCount; ii++) {//USDEUR * USDGBP * USDJPY * USDCHF
      if(currName != currensyName[ii]){
         opPrice  = 0.0;
         symbol1 = StringConcatenate(currName, currensyName[ii]);
         symbol2 = StringConcatenate(currensyName[ii], currName);
         if (indiUtils.hasSymbol(symbol1)) {
             opPrice = indiUtils.getPrice(symbol1, shift);
         } else if(indiUtils.hasSymbol(symbol2)) {
             opPrice = indiUtils.getPrice(symbol2, shift, true);
         } else {  //    ,   USD
            symbol1 = StringConcatenate(currName,         "USD");
            symbol2 = StringConcatenate("USD",            currensyName[ii]);
            symbol3 = StringConcatenate("USD",            currName);
            symbol4 = StringConcatenate(currensyName[ii], "USD");//EURJPY (EUR / USD) * (USD / JPY) =  EUR / JPY
            if (indiUtils.hasSymbol(symbol1) && indiUtils.hasSymbol(symbol2)) {
               opPrice = indiUtils.getPrice(symbol1, shift) * indiUtils.getPrice(symbol2, shift);
            } else if(indiUtils.hasSymbol(symbol3) && indiUtils.hasSymbol(symbol4)) {//JPYEUR   (JPY / USD) * (USD / EUR) =  JPY / EUR    
               opPrice = indiUtils.getPrice(symbol3, shift, true) * indiUtils.getPrice(symbol4, shift, true);
            } else {
               opPrice = EMPTY_VALUE;
            }
         }
       
         if (opPrice == EMPTY_VALUE){
            return(EMPTY_VALUE);
         } else {
            res *= opPrice;
         }
      }
   }
   return(MathPow(res, 1.0 / currCount));
}