//+------------------------------------------------------------------+
//|                                   Copyright  2010, Ivan Kornilov|
//|                                            indicators\objects.mqh|
//|                                   excelf@gmail.com, skype: excelf|
//+------------------------------------------------------------------+
//#include <indicators\objects.mqh>
color  objects.labelColor = Black;
string objects.font       = "Calibri";
bool objects.drowArrow = false;

string objects.orderLine(int ticket, double price, datetime beginTime, datetime endTime, bool takeProfit) {
   if(takeProfit) {
      string name = "takeProfitLine";
      color col = Green;
   } else {
      name = "stopLoseLine";
      col = Red;
   }
   name = StringConcatenate(ticket, "_", name, "_", TimeToStr(beginTime, TIME_DATE|TIME_MINUTES));
   
   
   ObjectCreate(name, OBJ_TREND, 0, beginTime, price, endTime, price);
   ObjectSet(name, OBJPROP_COLOR, col);
   ObjectSet(name, OBJPROP_BACK, false);
   ObjectSet(name, OBJPROP_RAY,  false);
   return(name);
}

double objects.getLinePrice(string name) {
   if(ObjectFind(name) == -1) {
      Print("    .");
      return(-1);
   }
   
   double price1 = ObjectGet(name, OBJPROP_PRICE1);
   datetime time1  = ObjectGet(name, OBJPROP_TIME1);
   double price2 = ObjectGet(name, OBJPROP_PRICE2);
   datetime time2  = ObjectGet(name, OBJPROP_TIME2);
   if(MathMin(time1, time2) > TimeCurrent() || MathMax(time1, time2) < TimeCurrent()) {  
      Print("    .");
      return(-1);
   }
   
   int shift1 = iBarShift(NULL, 0, time1, true);
   int shift2 = iBarShift(NULL, 0, time2, true);
   int shift = MathMax(shift1, shift2);
   //Print("shift1: " + shift1 + " name: " + name +  " time1: " + TimeToStr(time1, TIME_DATE|TIME_MINUTES));
   //Print("shift2: " + shift2 + " name: " + name +  " time2: " + TimeToStr(time2, TIME_DATE|TIME_MINUTES));
   datetime timeNow  = TimeCurrent();
 
   
   double priceSize = price2 - price1;
   int    lineSize  = MathAbs(time2 - time1)/ Period() / 60;
   //Print("lineSize: " + lineSize);

   double price = price1 + priceSize / lineSize  * shift;
   
   //Print("objects.getLinePrice shift1 " + shift1);
   //Print("objects.getLinePrice shift2 " + shift2);
   //Print("objects.getLinePrice price "  + price);


   return(price);
}



void objects.createDayLevel(string name, double level, datetime openTime, datetime closeTime, int width, color lineColor) {
    if(ObjectFind(name) != -1) {
        ObjectDelete(name);
    }
    name = name + TimeDayOfYear(openTime);
    /*
    if(TimeDayOfWeek(closeTime) == 0 || TimeDayOfWeek(closeTime) == 6) {
        closeTime = closeTime + 60 * 60 * 24 * 2;
    }*/
    ObjectCreate(name, OBJ_TREND, 0, openTime, level, closeTime, level);
    ObjectSet(name, OBJPROP_COLOR, lineColor);
    ObjectSet(name, OBJPROP_WIDTH, width);
    ObjectSet(name, OBJPROP_BACK, true);
    ObjectSet(name, OBJPROP_RAY, false);
}

void objects.verlicalLine(datetime time) {
   string name = "verlicalLine_" + TimeToStr(time, TIME_DATE|TIME_MINUTES);
   ObjectCreate(name, OBJ_VLINE, 0, time, 0);
   ObjectSet(name, OBJPROP_COLOR, Red);
   ObjectSet(name, OBJPROP_BACK, true);
   ObjectSet(name, OBJPROP_WIDTH, 2);
}

void objects.makeLabel(string name, int x, int y, string txt){
   ObjectCreate(name, OBJ_LABEL,0, 0, 0);
   ObjectSet(name, OBJPROP_CORNER, 0);
   ObjectSetText(name, txt, 12, objects.font, objects.labelColor);
   ObjectSet(name, OBJPROP_XDISTANCE, x);
   ObjectSet(name, OBJPROP_YDISTANCE, y);
}

void objects.drawTriangle(datetime openTime, double openPrice, datetime closeTime, double closePrice, int signal){
    if(signal == SIGNAL_NONE) {
        return;
    }
    string name = "triangle_" + TimeToStr(openTime, TIME_DATE|TIME_MINUTES);
    ObjectCreate(name, OBJ_TRIANGLE, 0, openTime, openPrice, closeTime, openPrice, closeTime, closePrice);
    int points;
    if(signal == SIGNAL_BUY) {
        points = (closePrice - openPrice) / Point;
    } else if(signal == SIGNAL_SELL){
        points = (openPrice - closePrice) / Point;
    }
    
    color triangleColor;
    if (points < 0){
        triangleColor = Red;
    } else {
        triangleColor = Green;
    }
    
    ObjectSet(name, OBJPROP_COLOR, triangleColor);
    ObjectSet(name, OBJPROP_BACK, false);
    ObjectSet(name, OBJPROP_WIDTH, 1);

    string text = "trianglePoints" + TimeToStr(openTime, TIME_DATE|TIME_MINUTES);
    ObjectCreate (text, OBJ_TEXT, 0, closeTime, closePrice - 20 * Point);
    ObjectSetText(text, DoubleToStr(MathFloor(points), 0), 10, objects.font, triangleColor);
}

void objects.setText(string name, string txt){
   ObjectSetText(name, txt, 12, objects.font, objects.labelColor);
}

void objects.deleteAll(string prefix) {
    string name;
    for(int i = ObjectsTotal(); i >= 0;i--) {
      name = ObjectName(i);
      if(StringFind(name, prefix, 0) == 0) {
         ObjectDelete(name);
      }
   }
}

#define LABEL_NAME "PointBalance"
#define LABEL_TEXT "Point Balance Total: "
int      objects.pointsTotal = 0;
datetime objects.time        = 0;
int      objects.signal      = 0;

void objects.addPointArrow(int signal, int i) {
   if(!objects.drowArrow) {
      return;
   }
   string name = StringConcatenate("arrow_", TimeToStr(Time[i], TIME_DATE|TIME_MINUTES));
   ObjectDelete(name);
   

   if(signal == 1 && (objects.signal == -1 || objects.signal == 0)) {
      ObjectCreate(name, OBJ_ARROW, 0, Time[i], Open[i], 0, 0, 0, 0);                
      ObjectSet(name, OBJPROP_ARROWCODE, 225);
      ObjectSet(name, OBJPROP_WIDTH, 1);
      ObjectSet(name, OBJPROP_COLOR, DeepSkyBlue);
   } else if(signal == -1 && (objects.signal == 1 || objects.signal == 0)) {
      ObjectCreate(name, OBJ_ARROW, 0, Time[i], Open[i], 0, 0, 0, 0);                
      ObjectSet(name, OBJPROP_ARROWCODE, 226);
      ObjectSet(name, OBJPROP_WIDTH, 1);
      ObjectSet(name, OBJPROP_COLOR, OrangeRed);
   }
   

   if(objects.signal != 0) {
      int signalOpenShift = indiUtils.barIdByTime(objects.time);
      int points = 0;
      if(objects.signal == -1) {
        points = (Close[signalOpenShift] - Close[i]) / Point;
      } else if(objects.signal == 1){
        points = (Close[i] - Close[signalOpenShift]) / Point;
      }
      objects.pointsTotal += (points - MarketInfo(Symbol(),MODE_SPREAD));
      //pointsTotal += points;
      objects.setText(LABEL_NAME, LABEL_TEXT + objects.pointsTotal);
   }
   objects.time = Time[i];
   objects.signal = signal;
} 



void objects.deinitArrawObjects() {
   if(objects.drowArrow) {
      objects.deleteAll("arrow_");
      ObjectDelete(LABEL_NAME);
   }
}

void objects.initArrawObjects() {
   if(objects.drowArrow) {
      objects.pointsTotal = 0;
      objects.makeLabel(LABEL_NAME, 10, 40, LABEL_TEXT);   
      objects.deleteAll("arrow_");
   }
}